#/bin/bash

# Script for filtered cube cell data exports in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
   -H "Content-Type:application/x-www-form-urlencoded" \
   -d "grant_type=password" \
   -d "client_id=${CLIENTID}" \
   -d "username=${USERNAME}" \
   -d "password=${PASSWORD}" \
   "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant and planning application
PLANNING_APP_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${API}/api/v1/tenants" | jq -r .tenants[0]._links.planningApplications[0].href)

# Navigation to cube
CUBES_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${PLANNING_APP_URL}" | jq -r ._links.allCubes.href)

CUBE_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${CUBES_URL}" | jq -r .cubes[0]._links.self.href)

# Full data download (shows data on console)
curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Accept:application/x.de.qvantum-plan.cube-cell-data" \
   "${CUBE_URL}/cells?inputAllowedFilter=ALL"

# Data download (with different dimension restrictions filters)
DIMENSION_RESTRICTIONS_FILTERS_PATH="$(pwd)/filters"

DIMENSION_RESTRICTIONS_JSONFILES="dimension-restrictions-leaves.json \
  dimension-restrictions-one-single.json \
  dimension-restrictions-multiple-single.json \
  dimension-restrictions-combined.json"

for file in ${DIMENSION_RESTRICTIONS_JSONFILES}
do

   filter_encoded=$(jq -r @uri <<< $(cat "${DIMENSION_RESTRICTIONS_FILTERS_PATH}/${file}"))

   echo "Filter JSON file: ${file}"
   echo "Filter encoded: ${filter_encoded}"

   result=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Accept:application/x.de.qvantum-plan.cube-cell-data" \
   "${CUBE_URL}/cells?dimensionRestrictions=${filter_encoded}")

   echo "${result}"
done

# Data download (with combined dimension restrictions and input allowed filters)
dimensionRestrictionsFilterEncoded=$(jq -r @uri <<< $(cat "${DIMENSION_RESTRICTIONS_FILTERS_PATH}/dimension-restrictions-combined.json"))
inputAllowedFilter="ALL"

curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Accept:application/x.de.qvantum-plan.cube-cell-data" \
   "${CUBE_URL}/cells?inputAllowedFilter=${inputAllowedFilter}&dimensionRestrictions=${dimensionRestrictionsFilterEncoded}"
