#/bin/bash

# Script for cube cell data uploads in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
   -H "Content-Type:application/x-www-form-urlencoded" \
   -d "grant_type=password" \
   -d "client_id=${CLIENTID}" \
   -d "username=${USERNAME}" \
   -d "password=${PASSWORD}" \
   "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant and planning application
PLANNING_APP_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${API}/api/v1/tenants" | jq -r .tenants[0]._links.planningApplications[0].href)

# Navigation to cube
CUBES_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${PLANNING_APP_URL}" | jq -r ._links.allCubes.href)

CUBE_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${CUBES_URL}" | jq -r .cubes[0]._links.self.href)

CUBE_DATA_UPLOAD_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${CUBE_URL}" | jq -r ._links.dataUploads.href)

# Asynchronous upload of cube cell data yields history entry URL for status polling
CUBE_DATA_PATH=$(pwd)/../csv/data/data.csv

HISTORY_ENTRY_URL=$(curl -s -i -X POST \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Content-Type:multipart/form-data" \
   -F "file=@${CUBE_DATA_PATH}; \
       type=application/x.de.qvantum-plan.cube-cell-data;" \
   "${CUBE_DATA_UPLOAD_URL}" | grep "Location:" | sed "s/Location: //g" | tr -d "[:space:]")

# Repeated status polling until processing is complete
UPLOAD_STATUS="PROCESSING"

while [[ ${UPLOAD_STATUS} == "PROCESSING" ]]
do
  # Check status of cube cell data upload
  UPLOAD_RESPONSE=$(curl -s -X GET \
    -H "Authorization:Bearer ${TOKEN}" \
    "${HISTORY_ENTRY_URL}")
  UPLOAD_STATUS=$(echo ${UPLOAD_RESPONSE} | jq .status -r)
  UPLOAD_IMPORT_PROGRESS=$(echo ${UPLOAD_RESPONSE} | jq .importProgress -r)
  UPLOAD_AGGREGATE_PROGRESS=$(echo ${UPLOAD_RESPONSE} | jq .aggregateProgress -r)
  echo "Progress: ${UPLOAD_IMPORT_PROGRESS} (import), ${UPLOAD_AGGREGATE_PROGRESS} (aggregation)"
done

# Result report
echo "Final cube cell data upload result:"
echo "${UPLOAD_RESPONSE}" | jq .
