#/bin/bash

# Script for dimension updates in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
   -H "Content-Type:application/x-www-form-urlencoded" \
   -d "grant_type=password" \
   -d "client_id=${CLIENTID}" \
   -d "username=${USERNAME}" \
   -d "password=${PASSWORD}" \
   "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant and planning application
PLANNING_APP_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
  "${API}/api/v1/tenants" | jq -r .tenants[0]._links.planningApplications[0].href)

# Navigation to cube
CUBE_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${PLANNING_APP_URL}" | jq -r ._links.allCubes.href)

# Navigation to particular dimension
DIMENSION_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${CUBE_URL}" | jq -r .cubes[0]._links.dimensions[0].href)

# Navigation to dimension aspect
ASPECT_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${DIMENSION_URL}" | jq -r ._links.aspect.href)

# Retrieve upload URL for dimension definition update
ASPECT_UPLOAD_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${ASPECT_URL}" | jq -r ._links.upload.href)

# Asynchronous upload of updated dimension definition yields history entry URL for status polling
CSVFILE="$(pwd)/../csv/metadata-update/dimensions/1.csv"
HISTORY_ENTRY_URL=$(curl -s -i -X POST \
  -H "Authorization:Bearer ${TOKEN}" \
  -F "file=@${CSVFILE};type=application/x.de.qvantum-plan.aspect-with-structure" \
  "${ASPECT_UPLOAD_URL}" | grep "Location:" | sed "s/Location: //g" | tr -d "[:space:]")

# Repeated status polling until processing is complete
UPLOAD_STATUS="PROCESSING"

while [[ ${UPLOAD_STATUS} == "PROCESSING" ]]
do
  # Check status of dimension update
  UPLOAD_RESPONSE=$(curl -s -X GET \
    -H "Authorization:Bearer ${TOKEN}" \
    "${HISTORY_ENTRY_URL}")
  UPLOAD_STATUS=$(echo ${UPLOAD_RESPONSE} | jq .status -r)
  echo "Upload Status: ${UPLOAD_STATUS}"
done

# Result report
echo "Final dimension update result:"
echo "${UPLOAD_RESPONSE}" | jq .
