#/bin/bash

# Script for forms definitions exports/imports in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
  -H "Content-Type:application/x-www-form-urlencoded" \
  -d "grant_type=password" \
  -d "client_id=${CLIENTID}" \
  -d "username=${USERNAME}" \
  -d "password=${PASSWORD}" \
  "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant and planning application
PLANNING_APP_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${API}/api/v1/tenants" | jq -r .tenants[0]._links.planningApplications[0].href)

# Navigation to cube
CUBES_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${PLANNING_APP_URL}" | jq -r ._links.allCubes.href)

CUBE_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${CUBES_URL}" | jq -r .cubes[0]._links.self.href)

KEY_FIGURES_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${CUBE_URL}" | jq -r ._links.keyFigures.href)

KEY_FIGURES_UPLOADS_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${CUBE_URL}" | jq -r ._links.keyFiguresUploads.href)

echo "Importing forms definition..."

# Asynchronous upload of key figures yields history entry URL for status polling
HISTORY_ENTRY_URL=$(curl -s -i -X POST \
  -H "Authorization:Bearer ${TOKEN}" \
  -H "Content-Type:application/x.de.qvantum-plan.external-key-figures+json" \
  -d @../json/key-figures-pretty.json \
  "${KEY_FIGURES_UPLOADS_URL}" | grep "Location:" | sed "s/Location: //g" | tr -d "[:space:]")

# Repeated status polling until processing is complete
UPLOAD_STATUS="PROCESSING"

while [[ ${UPLOAD_STATUS} == "PROCESSING" ]]
do
  # Check status of key figures upload
  UPLOAD_RESPONSE=$(curl -s -X GET \
    -H "Authorization:Bearer ${TOKEN}" \
    "${HISTORY_ENTRY_URL}")
  UPLOAD_STATUS=$(echo ${UPLOAD_RESPONSE} | jq .status -r)
  echo "Upload Status: ${UPLOAD_STATUS}"
done

# Result report
echo "Final key figures upload result:"
echo "${UPLOAD_RESPONSE}" | jq .

echo "Exporting current key figures..."

curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  -H "Accept:application/x.de.qvantum-plan.external-key-figures+json" \
  "${KEY_FIGURES_URL}" | jq -r .

