#/bin/bash

# Script for planning model definition uploads in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
   -H "Content-Type:application/x-www-form-urlencoded" \
   -d "grant_type=password" \
   -d "client_id=${CLIENTID}" \
   -d "username=${USERNAME}" \
   -d "password=${PASSWORD}" \
   "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant and planning application
PLANNING_APP_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
  "${API}/api/v1/tenants" | jq -r .tenants[0]._links.planningApplications[0].href)

# Navigation to cube
CUBES_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${PLANNING_APP_URL}" | jq -r ._links.allCubes.href)

MODEL_UPDATE_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
   "${CUBES_URL}" | jq -r .cubes[0]._links.modelUploads.href)

# Planning model upload
MODEL_META_PATH=$(pwd)/../csv/metadata-update
DIM_META_PATH=${MODEL_META_PATH}/dimensions
PLANNING_MODEL_PATH=${MODEL_META_PATH}/planning-model.csv

dim_files=(${DIM_META_PATH}/*.csv)
dim_file_params=()
for dim_file in "${dim_files[@]}"
do
  num=$(basename $dim_file .csv)
  dim_file_params+=(-F "${num}=@${dim_file};type=text/csv")
done

# Asynchronous upload of planning model update yields history entry URL for status polling
HISTORY_ENTRY_URL=$(curl -s -i -X POST \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Content-Type:multipart/form-data" \
   -F "cubeKey=1" \
   -F "cubeLabel=Cube" \
   -F "planning-model=@${PLANNING_MODEL_PATH};type=text/csv" \
   "${dim_file_params[@]}" \
 "${MODEL_UPDATE_URL}" | grep "Location:" | sed "s/Location: //g" | tr -d "[:space:]")

# Repeated status polling until processing is complete
UPLOAD_STATUS="PROCESSING"

while [[ ${UPLOAD_STATUS} == "PROCESSING" ]]
do
  # Check status of planning model update upload
  UPLOAD_RESPONSE=$(curl -s -X GET \
    -H "Authorization:Bearer ${TOKEN}" \
    "${HISTORY_ENTRY_URL}")
  UPLOAD_STATUS=$(echo ${UPLOAD_RESPONSE} | jq .status -r)
  echo "Upload Status: ${UPLOAD_STATUS}"
done

# Result report
echo "Final planning model update result:"
echo "${UPLOAD_RESPONSE}" | jq .
