#/bin/bash

# Script for user definition upload and retrieval in QVANTUM, based on curl and jq.
# This script is strongly simplified, as it completely omits error processing.

# Endpoints for QVANTUM tenants
AUTH=https://auth.qvantum-plan.de
API=https://api.qvantum-plan.de

# TODO: fill in your tenant information
#TENANT=
#CLIENTID=
#USERNAME=
#PASSWORD=

# Authentication
TOKEN=$(curl -s -X POST \
   -H "Content-Type:application/x-www-form-urlencoded" \
   -d "grant_type=password" \
   -d "client_id=${CLIENTID}" \
   -d "username=${USERNAME}" \
   -d "password=${PASSWORD}" \
   "${AUTH}/auth/realms/${TENANT}/protocol/openid-connect/token" | jq -r .access_token)

# Navigation to tenant
TENANT_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
  "${API}/api/v1/tenants" | jq -r .tenants[0]._links.self.href)

USER_UPLOAD_URL=$(curl -s -X GET \
   -H "Authorization:Bearer ${TOKEN}" \
  "${TENANT_URL}" | jq -r ._links.userUploads.href);

# User definition upload
USER_DEFINITION_PATH=$(pwd)/../csv/metadata/team/users.csv

# Asynchronous upload of user definition yields history entry URL for status polling
HISTORY_ENTRY_URL=$(curl -s -i -X POST \
   -H "Authorization:Bearer ${TOKEN}" \
   -H "Content-Type:multipart/form-data" \
   -F "file=@${USER_DEFINITION_PATH};type=text/csv" \
 "${USER_UPLOAD_URL}" | grep "Location:" | sed "s/Location: //g" | tr -d "[:space:]")

# Repeated status polling until processing is complete
UPLOAD_STATUS="PROCESSING"

while [[ ${UPLOAD_STATUS} == "PROCESSING" ]]
do
  # Check upload processing status
  UPLOAD_RESPONSE=$(curl -s -X GET \
    -H "Authorization:Bearer ${TOKEN}" \
    "${HISTORY_ENTRY_URL}")
  UPLOAD_STATUS=$(echo ${UPLOAD_RESPONSE} | jq .status -r)
  echo "Upload Status: ${UPLOAD_STATUS}"
done

# Result report
echo "Final user definition upload result:"
echo "${UPLOAD_RESPONSE}" | jq .

# User definition retrieval
USERS_URL=$(curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${TENANT_URL}" | jq -r ._links.allUsers.href | sed -e "s/{.*}//g");

SHOW_USERS_URL="${USERS_URL}?includeControllers=false"

curl -s -X GET \
  -H "Authorization:Bearer ${TOKEN}" \
  "${SHOW_USERS_URL}"
